;********************************
;*		ADC Wert einlesen		*
;********************************
sample_adc:
    		sbi     ADCSRA, ADSC        ; den ADC Wandlung starten
 
wait_adc:
    		sbic    ADCSRA, ADSC        ; wenn der ADC fertig ist, wird dieses Bit gelscht
    		rjmp    wait_adc
 
; ADC einlesen:
     		in      w, ADCL         	; immer zuerst LOW Byte lesen
    		in      w, ADCH 			; danach das mittlerweile gesperrte High Byte
										; beides in w einlesen, da nur 8 bit ntig
			st		z+,w				; wert im ram ablegen
			cpi		zl,$68				; zeiger nach 8 wandlungen wieder auf ramanfang
			brne	adc_ende			; wenn nicht adc_ende
mittelwert:	clr		yh					; ansonsten neuen mittelwert errechnen
			ld		yl,-z
			ld		w,-z
			add		yl,w				; w zu y pointer addieren (zph ist null -> nur
			adc		yh,zh				; der bertrag wird zu yph addiert
			cpi		zl,$60				; alle werte addiert?
			brne	PC-4				; wenn nicht weitermachen
			ror		yh					; yp 3x nach rechts verschieben (durch 8 teilen)
			ror		yl
			ror		yh
			ror		yl
			ror		yh
			ror		yl					; hier ist der neue mittelwert
			mov		w,yl				; mittelwert in w zurckgeben			
adc_ende:	ret



;********************************
;*		ADC init				*
;********************************
; ADC initialisieren: Single Conversion, Vorteiler 128
 
init_adc_m8:
		    ldi     w,0b01100001	    ;  ADMUX:
    		out     ADMUX, w			; +-------+-------+-------+-------+-------+-------+-------+-------+
										; | REFS1 | REFS0 | ADLAR |       |  MUX3 |  MUX2 |  MUX1 |  MUX0 |
 										; +-------+-------+-------+-------+-------+-------+-------+-------+
 
										; Referenzspannung REFS1, REFS0
										; 0	0	externe Referenz
										; 0	1	interne Referenz: Avcc
										; 1	0	wird beim Mega8 nicht benutzt
										; 1	1	interne Referenz: 2.56 Volt
										; Ausrichtung ADLAR	
										; 0	Das Ergebnis wird in den Registern ADCH/ADCL rechtsbndig ausgerichtet. 
										;	Die 8 niederwertigsten Bits des Ergebnisses werden in ADCL abgelegt. 
										;	Die verbleibenden 2 Bits des Ergebnisses werden im Register ADCH in den Bits 0 und 1 abgelegt.
										; 1	Das Ergebnis wird in den Registern ADCH/ADCL linksbndig ausgerichtet. 
										;	Die 8 hchstwertigen Bits des Ergebnisses werden in ADCH abgelegt. 
										;	Die verbleibenden 2 niederwertigen Bits werden im Register ADCL in den Bits 6 und 7 abgelegt.
										; Kanalwahl MUX3, MUX2, MUX1, MUX0
										;	0	0	0	0	Kanal 0, Pin PC0
										;	0	0	0	1	Kanal 1, Pin PC1
										;	0	0	1	0	Kanal 2, Pin PC2
										;	0	0	1	1	Kanal 3, Pin PC3
										;	0	1	0	0	Kanal 4, Pin PC4
										;	0	1	0	1	Kanal 5, Pin PC5
										;	1	1	1	0	1.23V, Vbg
										;	1	1	1	1	0V, GND

    		ldi     w,0b10000110		;  ADCSRA:
    		out     ADCSRA, w			; +-------+-------+-------+-------+-------+-------+-------+-------+
 										; |  ADEN |  ADSC |  ADFR |  ADIF |  ADIE | ADPS2 | ADPS1 | ADPS0 |
 										; +-------+-------+-------+-------+-------+-------+-------+-------+
										; ADEN (ADC Enable): Mittels ADEN wird der ADC ein und ausgeschaltet.
										;	Eine 1 an dieser Bitposition schaltet den ADC ein. 
										; ADSC (ADC Start Conversion): Wird eine 1 an diese Bitposition geschrieben,
										;	so beginnt der ADC mit der Wandlung. Das Bit bleibt auf 1, solange die Wandlung im Gange ist.
										;	Wenn die Wandlung beendet ist, wird dieses Bit von der ADC Hardware wieder auf 0 gesetzt. 
										; ADFR (ADC Free Running): Wird eine 1 an ADFR geschrieben,
										;	so wird der ADC im Free Running Modus betrieben. Dabei startet der ADC nach dem Abschluss
										;	einer Messung automatisch die nchste Messung. Die erste Messung wird ganz normal
										;	ber das Setzen des ADSC Bits gestartet. 
										; ADIF (ADC Interrupt Flag): Wenn eine Messung abgeschlossen ist, 
										;	wird das ADIF Bit gesetzt. Ist zustzlich noch das ADIE Bit gesetzt,
										;	so wird ein Interrupt ausgelst und der entsprechende Interrupt Handler angesprungen. 
										; ADIE (ADC Interrupt Enable): Wird eine 1 an ADIE geschrieben,
										;	so lst der ADC nach Beendigung einer Messung einen Interrupt aus. 
										; ADPS2, ADPS1, ADPS0 (ADC Prescaler): Mit dem Prescaler kann die ADC-Frequenz gewhlt werden.
										;	Laut Datenblatt sollte diese fr die optimale Auflsung zwischen 50KHz und 200kHz liegen. 
										;	Ist die Wandlerfrequenz langsamer eingestellt, kann es passieren dass die eingebaute Sample & Hold 
										;	Schaltung die Eingangsspannung nicht lange genug konstant halten kann. Ist die Frequenz aber zu schnell 
										;	eingestellt, dann kann es passieren dass sich die Sample & Hold Schaltung nicht schnell genug an die 
										;	Eingangsspannung anpassen kann.
										;	ADPS2	ADPS1	ADPS0	Vorteiler
										;	0	0	0	2
										;	0	0	1	2
										;	0	1	0	4
										;	0	1	1	8
										;	1	0	0	16
										;	1	0	1	32
										;	1	1	0	64
										;	1	1	1	128
			ret
