;************************************************************************************************
;		EEProm Daten in die Arbeitsregister laden												*
;************************************************************************************************
;EEPROMAddressen
.equ	EEAdresse	=	0			; EEPROM Adressen der CV Register
.equ	EEVstart	=	1
.equ	EEVinc		=	2
.equ	EEVdec		=	3
.equ	EEVmax		=	4
.equ	EEPWMperi	=	8
.equ	EElanghigh	=	16
.equ	EElanglow	=	17
.equ	EECV29		=	28
.equ	EEdimmer	=	49
.equ	EEVmerken	=	50			; Bit0-6 128 Fahrstufen Bit7 Richtung
.equ	EEkickZeit	=	64
.equ	EEkickFS	=	65
.equ	EEkennl		=	66			; CV67-CV94 -> 66-93

;Registerdef.
;.def 	Adresse 	= 	r1			; eigene Adresse bzw bei langen Adressen der high wert
;.def 	PWMein	 	= 	r4			; enthlt TCCR info mit aktuelle PWM frequenz wenn PWM ein
;.def	Dimmer		=	r8			; Licht - Dimmer
;.def 	w		 	= 	r17			; Arbeitsregister
;.def 	w2		 	= 	r22			; Arbeitsregister 2
;.def	eead		=	r23			; eepromaddresse

;Daten def.

.DSEG

;EMK:				.byte 8 ; 8 Byte fr die letzten EMK Werte reservieren (ElektroMotorischeKraft)
;AdresseLow:		.byte 1	; fr lange Adresse LowByte 1 Byte im SRAM reservieren
;V_start:			.byte 1	; fr V_start 1 Byte im SRAM reservieren
;IncWert:			.byte 1	; fr Beschleunigungswert 1 Byte im SRAM reservieren
;DecWert:			.byte 1	; fr Verzgerungswert 1 Byte im SRAM reservieren
;V_max:				.byte 1	; fr V_max 1 Byte im SRAM reservieren
;KickZeit:			.byte 1 ; fr Kickstartzeit 1 Byte im SRAM reservieren
;KickFS:			.byte 1 ; fr Kickstart Fahrstufe im SRAM reservieren
;Kennline:			.byte 28; fr Fahrstufen Kennline 28 Byte reservieren

.CSEG

		ldi		eead,EEAdresse		; Hole Dekoder Adresse (kurze Adresse)
		rcall	eepromlesen
		mov		Adresse,w
		ldi		eead,EEVstart		; Hole Startspannung
		rcall	eepromlesen
	  	sts 	V_start,w
		ldi		eead,EEVinc			; Hole Beschleunigung
		rcall	eepromlesen
	  	sts 	IncWert,w
		ldi		eead,EEVdec			; Hole Verzgerung
		rcall	eepromlesen
	  	sts 	DecWert,w
		ldi		eead,EEVmax			; Hole Maximalspannung
		rcall	eepromlesen
	  	sts 	V_max,w
		ldi		eead,EEDimmer		; Hole LichDimmer
		rcall	eepromlesen
	  	mov 	Dimmer,w
		ldi		eead,EEKickZeit		; Hole Kickstartzeit
		rcall	eepromlesen
	  	sts 	KickZeit,w	
		ldi		eead,EEKickFS		; Hole Kickstartfahrstufe
		rcall	eepromlesen
	  	sts 	KickFS,w			
		ldi		yl,low(kennline)	; Hole Kennline
		ldi		yh,high(kennline)
		ldi		eead,EEkennl
		rcall	eepromlesen
	  	st	 	y+,w
		inc		eead
		cpi		eead,EEkennl+28		; alle Werte eingelesen?
		brne	PC-4				; wenn nicht nchsten holen
		
		ldi 	eead,EEcv29			; Konfig bits holen 
	  	rcall 	eepromlesen
		mov		DCCreg,w
		cbr		DCCreg,(1<<FL)		; zur Sicherheit lichtbit abschalten (kommt nicht aus CV29)

		sbrs	DCCreg,erwAdresse	; wenn erweiterte (lang) Adresse an 1x springen
		rjmp	PC+7				; lange adresse holen berspringen
		ldi		eead,EElanghigh		; erweiterte Adresse holen
		rcall	eepromlesen
		mov		Adresse,w
		ldi		eead,EElanglow
		rcall	eepromlesen
		sts		AdresseLow,w
		
		sbrs	DCCreg,Vmerken		; wenn Funktion letzte Geschw. merken an 1x springen
		rjmp	PC+12				; V merken holen berspringen
		ldi		eead,EEVmerken		; erweiterte Adresse holen
		rcall	eepromlesen
		bst		w,7					; richtung holen (steht in bit7)
		bld		DCCreg,Richtung		; und ablegen
		sbrs	DCCReg,richtung		; dann setze RichtungsPort
		cbi		Portb,MotR		
		sbrc	DCCReg,richtung
		sbi		Portb,MotR
		cbr		w,0b10000000		; danach Richtungsbit aus v_ist entfernen
		mov		V_ist,w				; und V_ist setzen
		mov		V_soll,w			; V_soll auch setzen

		ldi		eead,EEPWMperi		; Hole Motor PWM frequenz
		rcall	eepromlesen			; und setzte damit Timer/Counter Control Register TCCR1B
		ldi		w2,0b00000001		; 8-bit PWM (bei 30Hz/122Hz/500Hz und 31KHz)
		out		TCCR1A,w2			; timer auf mode5 wgm12=1 wgm10=1 setzen
									; (Variable eead hier als w2 genutzt)
		cpi		w,0					; dann je nach CV PWM Frequenz setzen
		brne	PC+3
		ldi		w2,0b00001101		; Teiler 1024 -> 7812,5 Tackte/sec. /256 = 30,5 Hz PWM Frequenz
		out		TCCR1B,w2
		cpi		w,1
		brne	PC+3
		ldi		w2,0b00001100		; oder Teiler  256 -> 31250 Tackte/sec. /256 = 122 Hz PWM Frequenz
		out		TCCR1B,w2
		cpi		w,2
		brne	PC+3
		ldi		w2,0b00001011		; oder Teiler  64 -> 125000 Tackte/sec. /256 = 488 Hz PWM Frequenz
		out		TCCR1B,w2
		cpi		w,3					
		brne	PC+6
		ldi		w2,0b00000010		; 9-Bit PWM (bei 16KHz)
		out		TCCR1A,w2
		ldi		w2,0b00001001		; oder Teiler 1 -> 8000000 Tackte/sec mit 9-Bit PWM = 16KHz
		out		TCCR1B,w2
		sbr		Flag,(1<<PWM9Bit)	; Flag auf 16KHz Betrieb setzen
		cpi		w,4
		brne	PC+3
		ldi		w2,0b00001001		; oder Teiler 1 -> 8000000 Tackte/sec. /256 = 31KHz PWM Frequenz
		out		TCCR1B,w2
	
