;************************************************************************************************
;* init der mega8 Hardware fr dcc lokdecoder 													*
;************************************************************************************************

;********************************
;*		m8 Timer0 init			*
;********************************
; FREI
init_timer0_m8:

; Timer/Counter0 Control Register	
; timer mit Teiler auf ck/1 starten
;		ldi		w,0b00000001		; +------+------+------+------+------+------+------+------+
;		out		TCCR0,w				; |  -   |   -  |  -   |   -  |   -  | CS02 | CS01 | CS00 |
									; +------+------+------+------+------+------+------+------+
									; Bits 2-0 CS02-0: Clock Select
									; CS02 CS01 CS00
									;	0	0	0 	No clock source (Timer/Counter stopped)
									; 	0 	0 	1 	clkI/O/(No prescaling)
									; 	0 	1 	0 	clkI/O/8 (From prescaler)
									; 	0 	1 	1 	clkI/O/64 (From prescaler)
									; 	1 	0 	0 	clkI/O/256 (From prescaler)
									; 	1 	0 	1 	clkI/O/1024 (From prescaler)
									; 	1 	1 	0 	External clock source on T0 pin. Clock on falling edge.
									; 	1 	1 	1 	External clock source on T0 pin. Clock on rising edge.	

; Timer/Counter0 Register
;		ldi		w,0
;		out		tcnt0,w
	  
; Timer/Counter Interrupt Mask Register TIMSK:
;		in		w,TIMSK				; holen da beim m8 fr alle 3 Timer zustndig
;		sbr		w,(1<<TOIE0)		; +------+------+------+------+------+------+------+------+
 ;		out		TIMSK,w				; |OCIE2 | TOIE2|TICIE1|OCIE1A|OCIE1B|TOIE1 |   -  | TOIE0|
									; +------+------+------+------+------+------+------+------+
									; TOIE0: Timer/Counter0, Overflow Interrupt Enable

; Timer/Counter0 Interrupt Flag Register TIFR: lsche Timer0 Interupt
;		in		w,TIFR				; holen da beim m8 fr alle 3 Timer zustndig
;		cbr		w,(1<<TOV0)			; +------+------+------+------+------+------+------+------+
; 		out		TIFR,w				; | OCF2 | TOV2 | ICF1 | OCF1A| OCF1B|TOV1  |   -  | TOV0 |
									; +------+------+------+------+------+------+------+------+
									; TOV0: Timer/Counter0, Overflow Flag
;		ret

;********************************
;*		Timer1 init				*
;********************************
; Timer1 initialisieren:

init_timer1_m8:	
; Timer/Counter1 Control Register TCCR1A -> 8bit fast pwm mode5 Ausgang noch getrennt
;*	wird beim EEpromload passend gesetzt	*
;		ldi		w,0b00000000	; +------+------+------+------+------+------+------+------+
; 		out		TCCR1A,w		; |COM1A1|COM1A0|COM1B1|COM1B0|FOC1A |FOC1B |WGM11 |WGM10 |
 								; +------+------+------+------+------+------+------+------+
								; COM1A1/0: Compare Output Mode for channel A
								; COM1B1/0: Compare Output Mode for channel B
								;	0 0 Normal port operation, OC1A/OC1B disconnected
								;	1 0 Clear OC1A/OC1B on Compare Match, set OC1A/OC1B at TOP(fast PWM Mode)
								;   1 1 Set OC1A/OC1B on Compare Match, clear OC1A/OC1B at TOP(fast PWM Mode)
								; FOC1A/B: Force Output Compare for channel A/B
								;	0 0 Normal port operation
								; WGM11/0: Waveform Generation Mode
								;	Mode	WGM13	WGM12	WGM11	WGM10	Timer/Counter			TOP		Update	TOV1
								;					(CTC1)	(PWM11)	(PWM10)	Mode					OCR1x
								;	0		0		0		0		0			Normal 				0xFFFF 	Immediate MAX
								;	1		0		0		0		1		PWM Phase Correct8-bit 	0x00FF 	TOP 	BOTTOM
								;	2 		0 		0 		1 		0 		PWM Phase Correct9-bit 	0x01FF 	TOP 	BOTTOM
								;	3 		0 		0 		1 		1 		PWM Phase Correct10-bit 0x03FF 	TOP 	BOTTOM
								;	4 		0 		1 		0 		0 		CTC 					OCR1A	Immediate MAX
								;	5 		0 		1 		0 		1 		Fast PWM8-bit 			0x00FF 	TOP 	TOP
								;	6 		0 		1 		1 		0 		Fast PWM9-bit 			0x01FF 	TOP 	TOP
								;	7 		0 		1 		1 		1 		Fast PWM10-bit 			0x03FF 	TOP 	TOP
								;	8 		1 		0 		0 		0 		PWM Phase&Freq Correct	ICR1 	BOTTOM 	BOTTOM
								;	9 		1 		0 		0 		1 		PWM Phase&Freq Correct	OCR1A 	BOTTOM 	BOTTOM
								;	10 		1 		0 		1 		0 		PWM Phase Correct 		ICR1 	TOP 	BOTTOM
								;	11 		1 		0 		1 		1 		PWM Phase Correct 		OCR1A 	TOP 	BOTTOM
								;	12 		1 		1 		0 		0 		CTC 					ICR1 	Immediate MAX
								;	13 		1 		1 		0 		1 		(Reserved)
								;	14 		1 		1 		1 		0 		Fast PWM 				ICR1 	TOP 	TOP
								;	15 		1 		1 		1 		1 		Fast PWM 				OCR1A 	TOP 	TOP
												
; Timer/Counter1 Control Register  TCCR1B
;*	wird beim EEPromload gesetzt	*
;		ldi		w,0b00001101		; +------+------+------+------+------+------+------+------+
;		out		TCCR1B,w			; | ICNC1| ICES1|      | WGM13| WGM12| CS12 | CS11 | CS10 |
									; +------+------+------+------+------+------+------+------+
									; CS12 - CS10   Clock SelectCS12	CS11	CS10	Bedeutung
									;	0	0	0	keine (Der Timer ist angehalten)
									;	0	0	1	Vorteiler: 1
									;	0	1	0	Vorteiler: 8
									;	0	1	1	Vorteiler: 64
									;	1	0	0	Vorteiler: 256
									;	1	0	1	Vorteiler: 1024
									;	1	1	0	Externer Takt vom Pin T1, fallende Flanke
									;	1	1	1	Externer Takt vom Pin T1, steigende Flanke
									; WGM13:12: Waveform Generation Mode siehe TCCR1A
									; ICES1   Input Capture Edge Select
									; ICNC1   Input Capture Noise Canceler

; Timer/Counter 1 TCNT1H and TCNT1L
		clr		w
		out		TCNT1H,w
		out		TCNT1L,w

; Output Compare Register 1 A OCR1AH and OCR1AL
		clr		w
		out		OCR1AH,w
		out		OCR1AL,w

; Output Compare Register 1 B OCR1BH and OCR1BL
		clr		w
		out		OCR1BH,w
		out		OCR1BL,w

; Input Capture Register 1 ICR1H and ICR1L
		clr		w
		out		ICR1H,w
		out		ICR1L,w

; Timer/Counter Interrupt Mask Register TIMSK:
		in		w,TIMSK				; holen da beim m8 fr alle 3 Timer zustndig
;		cbr		w,(1<<TICIE1)		; +------+------+------+------+------+------+------+------+
 		out		TIMSK,w				; | OCIE2| TOIE2|TICIE1|OCIE1A|OCIE1B| TOIE1|      | TOIE0|
									; +------+------+------+------+------+------+------+------+
									; OCIE2: Timer/Counter2, Output Compare Match Interrupt Enable
									; TOIE2: Timer/Counter2, Overflow Interrupt Enable
									; TICIE1: Timer/Counter1, Input Capture Interrupt Enable
									; OCIE1A: Timer/Counter1, Output Compare A Match Interrupt Enable
									; OCIE1B: Timer/Counter1, Output Compare B Match Interrupt Enable
									; TOIE1: Timer/Counter1, Overflow Interrupt Enable
									; TOIE0: Timer/Counter0, Overflow Interrupt Enable

; Timer/Counter Interrupt Flag Register TIFR:
		in		w,TIFR				; holen da beim m8 fr alle 3 Timer zustndig
;		cbr		w,(1<<????)			; +------+------+------+------+------+------+------+------+
 		out		TIFR,w				; | OCF2 | TOV2 | ICF1 |OCF1A |OCF1B |TOV1  |      | TOV0 |
									; +------+------+------+------+------+------+------+------+
									; OCF2: Timer/Counter2, Output Compare Match Flag
									; TOV2: Timer/Counter2, Overflow Flag
									; ICF1: Timer/Counter1, Input Capture Flag
									; OCF1A: Timer/Counter1, Output Compare A Match Flag
									; OCF1B: Timer/Counter1, Output Compare B Match Flag
									; TOV1: Timer/Counter1, Overflow Flag
									; TOV0: Timer/Counter0, Overflow Flag
		ret

;********************************
;*		m8 Timer2 init			*
;********************************
; Timer2 macht alle 10us ein interrupt zum DCC Pin einlesen
; dazu Timer mit ck im CTC mode bis 80 zhlen lassen
init_timer2_m8:

; Timer/Counter2 Register
		clr		w
		out		TCNT2,w

; Output Compare Register			; mit 0 bis 79 -> 80 Tackte laden
		ldi		w,79
		out		OCR2,w

; Timer/Counter2 Control Register	
; timer mit Teiler auf ck/1 starten, CTC Mode, OC2 nicht verbunden
		ldi		w,0b00001001		; +------+------+------+------+------+------+------+------+
		out		TCCR2,w				; | FOC2 | WGM20| COM21| COM20| WGM21| CS02 | CS01 | CS00 |
									; +------+------+------+------+------+------+------+------+
									; Bit 7 FOC2: Force Output Compare
									; Bit 6,3 WGM21-0: Waveform Generation Mode
									; Mode	WGM21	WGM20	Mode		TOP		UpdateOCR2	TOV2FlagSet
									;	0	0		0		Normal 		0xFF 	Immediate	MAX
									;	1 	0 		1 		PWM,PhCo	0xFF 	TOP 		BOTTOM
									;	2 	1 		0 		CTC 		OCR2 	Immediate 	MAX
									;	3 	1 		1 		Fast PWM 	0xFF 	TOP 		MAX
									; Bit 5-4 COM21-0: Compare Match Output Mode
									; 0/0 -> Normal port operation, OC2 disconnected
									; Bits 2-0 CS02-0: Clock Select
									; CS02 CS01 CS00
									;	0	0	0 	No clock source (Timer/Counter stopped)
									; 	0 	0 	1 	clkI/O/(No prescaling)
									; 	0 	1 	0 	clkI/O/8 (From prescaler)
									; 	0 	1 	1 	clkI/O/32 (From prescaler)
									; 	1 	0 	0 	clkI/O/64 (From prescaler)
									; 	1 	0 	1 	clkI/O/128 (From prescaler)
									; 	1 	1 	0 	clkI/O/256 (From prescaler)
									; 	1 	1 	1 	clkI/O/1024 (From prescaler)	

; Asynchronous Status Register ASSR nur fr Takt ber externen keramikschwinger wichtig 
		ldi		w,0b00000000		; +------+------+------+------+------+------+------+------+
		out		ASSR,w				; |  -   |   -  |   -  |   -  | AS2  |TCN2UB|OCR2UB|TCR2UB|
									; +------+------+------+------+------+------+------+------+
									; Bit 3 AS2: Asynchronous Timer/Counter2 0 = clkI/O
									; Bit 2 TCN2UB: Timer/Counter2 Update Busy
									; Bit 1 OCR2UB: Output Compare Register2 Update Busy
									; Bit 0 TCR2UB: Timer/Counter Control Register2 Update Busy

; Timer/Counter Interrupt Mask Register TIMSK:
		in		w,TIMSK				; holen da beim m8 fr alle 3 Timer zustndig
		sbr		w,(1<<OCIE2)		; +------+------+------+------+------+------+------+------+
 		out		TIMSK,w				; |OCIE2 | TOIE2|TICIE1|OCIE1A|OCIE1B|TOIE1 |   -  | TOIE0|
									; +------+------+------+------+------+------+------+------+
									; Bit 7 OCIE2: Timer/Counter2 Output Compare Match Interrupt Enable
									; Bit 6 TOIE2: Timer/Counter2 Overflow Interrupt Enable

; Timer/Counter Interrupt Flag Register TIFR: lsche Timer2 OCF Interuptflag
		in		w,TIFR				; holen da beim m8 fr alle 3 Timer zustndig
		cbr		w,(1<<OCF2)			; +------+------+------+------+------+------+------+------+
 		out		TIFR,w				; | OCF2 | TOV2 | ICF1 | OCF1A| OCF1B|TOV1  |   -  | TOV0 |
									; +------+------+------+------+------+------+------+------+
									; Bit 7 OCF2: Output Compare Flag 2
									; Bit 6 TOV2: Timer/Counter2 Overflow Flag

; Special Function IO Register SFIOR
		clr		w					; +------+------+------+------+------+------+------+------+
		out		SFIOR,w				; |  -   |   -  |   -  |   -  | ACME | PUD  | PSR2 |PSR10 |
									; +------+------+------+------+------+------+------+------+
									; Bit 1 PSR2: Prescaler Reset Timer/Counter2
		ret
