;********************************************************************************************************************
;* mega8 dcc lokdecoder lastgeregelt																				*
;*																													*
;*								   ************   ***********														*
;*		RESET					1  * RESET/PC6 *** ADC5/PC5 * 28													*
;*		RXD-TEST-PC				2  * RXD/PD0	   ADC4/PC4 * 27													*
;*		TCD_TEST-PC				3  * TXD/PD1	   ADC3/PC3 * 26													*
;*      DCC-Eingang	  			4  * INT0/PD2	   ADC2/PC2 * 25													*
;*		F1						5  * INT1/PD3	   ADC1/PC1 * 24													*
;*		F2						6  * TO/PD4		   ADC0/PC0 * 23		Motor AD Wandler							*
;*		+5V UB					7  * VCC				GND * 22													*
;*		GND						8  * GND			   AREF * 21													*
;*		XTAL 8 MHz				9  * XTAL1/PB6		   AVCC	* 20													*
;*		XTAL					10 * XTAL2/PB7		SCK/PB5	* 19		ISP											*
;*		F3						11 * T1/PD5		   MISO/PB4 * 18		ISP											*
;*		F4						12 * AIN0/PD6	   MOSI/PB3 * 17		ISP											*
;*		Licht rck				13 * AIN1/PD7	   OC1B/PB2 * 16		Motor Richtung									*
;*		Licht vor				14 * ICP1/PB0	   OC1A/PB1 * 15		Motor PWM									*
;*								   **************************														*
;*																													*
;********************************************************************************************************************

.include "m8def.inc"

;********************************
;* FUSE info: fr 8MHz Quarz	*
;*								*
;* RSTDISBL:		nein(1)		*			;RESET Pin mit RESET Funktion
;* WDTON:			nein(1)		*			;WDT beim Start aus
;* SPIEN:			ja(0)		*			;serial programmieren erlaubt
;* CKOPT:			nein(1)		*			;
;* EESAVE:			nein		*			;eeprom wird bei chiperase auch gelscht
;* BOOTSZ0:			ja			*			;BOOTSZ legt Bootloaderbereich fest 11 = 1024 words
;* BOOTSZ1:			ja			*
;* BOOTRST:			nein(1)		*			;Reset Vektor auf 0x0000 (kein Bootloader)
;* BODLEVEL:		ja(0)		*			;BOD bei ca 4V VCC
;* BODEN:			ja(0)		*			;BOD ein
;* SUT1:			ja(0)		*			;SUT 01 = Startuptime BOD ein
;* SUT0:			nein(1)		*
;* CKSEL3:			nein(1)		*			;CKSEL 1111 = 8 MHz externer Quarz
;* CKSEL2:			nein(1)		*
;* CKSEL1:			nein(1)		*
;* CKSEL0:			nein(1)		*
;******************************** 

;************************************************************************************************
;* 						Konstantendef.															*
;************************************************************************************************

;Ports
.equ	MotR		=	2			; PB2 Motor Richtung Ausgang	
.equ	MotPWM		=	1			; PB1 Motor PWM Ausgang
.equ	LichtV		=	7			; PD7 Licht vorwrts
.equ	LichtR		=	0			; PB0 Licht rckwrts
.equ	F1			=	3			; PD3 Funktionsausgang 1
.equ	F2			=	4			; PD4 Funktionsausgang 2
.equ	F3			=	5			; PD5 Funktionsausgang 3
.equ	F4			=	6			; PD6 Funktionsausgang 4

.equ	fosc		= 	8000000		; Systemtakt in Hz
.equ 	BAUD  		= 	9600        ; Die gewuenschte Baudrate

.equ	UBRR_wert 	= (fosc/(16 * Baud))-1	;UBRR-Wert berechnen

;Flagregister
.equ	resetbit	=	0			; neues Byte abgelegt
.equ	frei1		=	1
.equ	frei2		=	2
.equ	frei3		=	3
.equ	bit			=	4
.equ	neuesbit	=	5
.equ	PWM9Bit		=	6			; 1 = PWM luft mit 9-Bit -> 16KHz
.equ	keinDCC		=	7			; fr analogbetrieb hier nicht aktiv

;DCCregister
.equ	Richtung_inv=	0			; 1= Fahrtrichtung invertiert
.equ	Fahrstufen	=	1			; 1= 28FS 0= 14FS
.equ	analog		=	2			; 0= nur DCC
.equ	Vmerken		=	3			; 1= merke letzten Fahrbefehl im EErpom
.equ	nutzeKennl	=	4			; 1= nutze Kennlinie
.equ	erwAdresse	=	5			; 1= nutze lange Adresse
.equ	FL			=	6			; Licht an
.equ	Richtung	=	7			; Fahrtrichtung wenn nicht inv. 1=vor 0=zurck


;************************************************************************************************
;* 						Registerdef.															*
;************************************************************************************************

.def 	Adresse 	= 	r1			; eigene Adresse bzw bei langen Adressen der high wert
.def 	V_IST 		= 	r2			; momentane Geschwindigkeit
.def 	V_SOLL	 	= 	r3			; Soll Geschwindigkeit
.def 	intsreg	 	= 	r4			; Kopie sreg im Interrupt (zum beschleunigen vom TIM2_ctc)
.def	FSzaehler	=	r5			; Fahrstufenzaehler zum dec/inc der Fahrstufen
.def 	Zaehler	 	= 	r6			; Vorzhler zum ausfhren vom Lichtdimmer, zhlt die 10us ctc interrups
.def 	reg7	 	= 	r7			; Register
.def 	Dimmer	 	= 	r8			; Licht - Dimmer
.def 	reg9	 	= 	r9			; Register
.def	byteA		=	r10			; Empfangsbytespeicher
.def	byteB		=	r11
.def	byteC		=	r12
.def	byteD		=	r13
.def	byteE		=	r14
.def	byteF		=	r15
.def	bitzaehler	=	r16			; Bitzaehler beim speichern
.def 	w		 	= 	r17			; Arbeitsregister
.def	flag		=	r18			; Flagregister
									;	Bit0 = 1 -> resetbit
									;	Bit1 = 1 -> frei
									;	Bit2 = 1 -> frei
									;	Bit3 = 1 -> Vsoll im EEProm merken
									;	Bit4 = 1 -> das neue bit
									;	Bit5 = 1 -> neues bit da	
									;	Bit6 = 1 -> PWM Pin ein im ctc interrupt
									;	Bit7 = 1 -> lnger als 10000us keine bits eingetroffen
.def 	DCCReg 		= 	r19			; DCC Register fr Funktionen und Fahrtrichtung
									;	Bit7 = 1 -> 28 Fahrstufen (0 = 14 Fahrstufen)
									;	Bit6 = 1 -> Fahrtrichtung reversiert
									;	Bit5 Fahrtrichtung 1=vorwrts, 0=rckwrts
									;   Bit4 
									;	Bit3  	
									;	Bit2 
									;	Bit1 
									;	Bit0 FL Licht ein/aus
.def 	Lowpegel	= 	r20			; Speicher fr Lowpegel
.def 	Highpegel 	= 	r21			; Speicher fr Highpegel
.def 	w2		 	= 	r22			; Arbeitsregister 2
.def	eead		=	r23			; eepromaddresse
.def	w3			=	r23			; oder Arbeitsregister 3
.def 	DimmerPWM 	= 	r24			; PWM Zaehler fr LichtDimmer
.def 	reg25	 	= 	r25			; Register
;.def 	xl		 	= 	r26			; 
;.def 	xh		 	= 	r27			; mit Hilfe des 10us CTC Interrupt
;.def 	yl		 	= 	r28			; 16 bit Arbeits/Zeigerregister
;.def 	yh		 	= 	r29			;
;.def 	zl		 	= 	r30			; zum speichern der ADC Werte im Ram
;.def 	zh		 	= 	r31			; 


;********************************
;*		Daten def.				*
;********************************
.DSEG

EMK:				.byte 8 ; 8 Byte fr die letzten EMK Werte reservieren (ElektroMotorischeKraft)
AdresseLow:			.byte 1	; fr lange Adresse LowByte 1 Byte im SRAM reservieren
V_start:			.byte 1	; fr V_start 1 Byte im SRAM reservieren
IncWert:			.byte 1	; fr Beschleunigungswert 1 Byte im SRAM reservieren
DecWert:			.byte 1	; fr Verzgerungswert 1 Byte im SRAM reservieren
V_max:				.byte 1	; fr V_max 1 Byte im SRAM reservieren
KickZeit:			.byte 1 ; fr Kickstartzeit 1 Byte im SRAM reservieren
KickFS:				.byte 1 ; fr Kickstart Fahrstufe im SRAM reservieren
Kennline:			.byte 28; fr Fahrstufen Kennline 28 Byte reservieren

;************************************************************************************************
;*  					ab hier Programmcode													*
;************************************************************************************************

.CSEG

;************************************
;* Reset- und Interrupt-Vektoren	*
;************************************

.ORG $0000
			rjmp 	RESET 			; Reset Handler
			reti; 	EXT_INT0 		; IRQ0 Handler
			reti; 	EXT_INT1 		; IRQ1 Handler
			rjmp 	TIM2_COMP 		; Timer2 Compare Handler
			reti;	TIM2_OVF 		; Timer2 Overflow Handler
			reti; 	TIM1_CAPT 		; Timer1 Capture Handler
			reti; 	TIM1_COMPA 		; Timer1 CompareA Handler
			reti; 	TIM1_COMPB 		; Timer1 CompareB Handler
			reti; 	TIM1_OVF 		; Timer1 Overflow Handler
			reti; 	TIM0_OVF 		; Timer0 Overflow Handler
			reti; 	SPI_STC 		; SPI Transfer Complete Handler
			reti; 	USART_RXC 		; USART RX Complete Handler
			reti; 	USART_UDRE 		; UDR Empty Handler
			reti; 	USART_TXC 		; USART TX Complete Handler
			reti; 	ADC 			; ADC Conversion Complete Handler
			reti; 	EE_RDY 			; EEPROM Ready Handler
			reti; 	ANA_COMP 		; Analog Comparator Handler
			reti; 	TWSI 			; Two-wire Serial Interface Handler
			reti; 	SPM_RDY 		; Store Program Memory Ready Handler

;************************************************************************************************
;*						Interruptprogramme														*
;************************************************************************************************
;************************************
;*		Timer_2_interrupt			*
;************************************
; 1. Teil DCC Pin abtasten:
; wird alle 10us ausgelst und liest dann bit von int0/d2 (auch jeder andere pin mglich) ein
; dazu teiler auf1 (8tackte=1us) -> bis 80 zhlen lassen und einen Compare interrupt auslsen
; zw 26 und 34 cpu tackte -> 25-30% cpu leistung werden hier verbraucht!
; es werden die register intsreg,lowpegel,highpegel belegt
; das ergebnis wird in den flags bit und neuesbit zurckgegeben
; 2. Teil Lichtdimmer und Fahrstufen setzen
; wird alle 2,56ms zustzlich zu Teil 1 ausgefhrt und setzt die Fahrtrichtungsbeleuchtung in 10 Dimmstufen
; und danach je nach gewnschter beschleunigungs/verzgerungs/kickstartrate die die MotorPWM neu gesetzt
; dadurch verlngert sich der interrupt um bis zu 38 cpu tackte -> im schlechtesten Fall bentigt der inter-
; rupt fast 90% der cpu leistung, aus diesem Grund werden die register nur gesichert, wenn sie auch 
; verndert werden

TIM2_COMP:		
		in		intsreg,sreg		; sreg sichern bewut nicht im fifo abgelegt, da so schneller

	  	sbic	pind,2				; wenn aktueller Pegel=0 1x springen
		inc		Highpegel			; sonst Highpegel inc
	  	sbis 	pind,2				; wenn aktueller Pegel=1 1x springen
	  	inc		Lowpegel			; sonst Lowpegel inc

	  	cpi 	Lowpegel,3			; wenn mindestens 3x Low eingetroffen
		brne	PC+2				; dann 1x springe
		clr		Highpegel			; sonst erst Highpegel lschen
		cpi		Highpegel,3			; ist jetzt Highpegel = 3
		brne	DCC_abtasten_ENDE	; wenn ungleich, dann fertig
		cbr		flag,(1<<bit)		; neues bit lschen
		cpi		Lowpegel,7			; ist Lowpegel kleiner 7
		brsh	PC+2				; wenn grer/gleich 1x springen
		sbr		flag,((1<<bit))		; neues bit setzen
		sbr		flag,(1<<neuesbit)	; flag neues bit da setzen
		clr		Lowpegel			; Lowpegel wieder lschen
		inc		Highpegel			; zur sicherheit 1x erhhen, damit beim nchste int nicht noch ein Byte erkannt wird
DCC_abtasten_ENDE:				
									; ab hier Licht Dimmer und Motor inc/dec setzen
		inc		Zaehler				; dazu Zaehler register alle 10us inc
		tst		Zaehler				; wenn nicht 1x umgelaufen = 0 (10us+256=2,56ms vergangen)
		brne	TIM2_ENDE			; interrupt zu ende
Licht_dimmer:						; wird alle 2,56ms ausgefhrt
		sbrs	DCCreg,FL			; wenn Licht an 1x springen
		rjmp	Licht_aus			; sonst gleich zu...
		inc		DimmerPWM			; inc
		cpi		DimmerPWM,11		; auf max testen
		brne	Di_ausschalten		; wenn nicht gehe zu..
		clr		DimmerPWM			; wenn max, auf 0 setzen
		sbrs	DCCreg,Richtung		; bei vorwrts 1x springen
		sbi		PortB,LichtR		; Lichtausgang rckwrts einschalten
		sbrc	DCCreg,Richtung		; wenn rckwrts 1x springen
		sbi		PortD,LichtV		; Lichtausgang vorwrts einschalten
		rjmp	Motor				; und weiter mit..
Di_ausschalten:
		cp		DimmerPWM,Dimmer	; auf Dimmerwert testen
		brne	Motor				; wenn ungleich gehe zu..
Licht_aus:
		cbi		PortB,LichtR		; wenn gleich, Ausgnge abschalten
		cbi		PortD,LichtV

Motor:				
		dec		FSzaehler			; Fahrstufenzaehler dec
		tst		FSzaehler			; auf Null testen
		brne	TIM2_Ende			; wenn ungleich ende
		cp		V_soll,V_ist		; auf beschleunigen/bremsen testen
		Breq	V_ist_konst			; wenn gleich gehe zu
		Brlo	V_bremsen			; wenn V_soll kleiner gehe zu..
V_beschleunigen:					; wenn V_soll groer hier weiter
		tst		V_ist				; ist V_ist=0 (Motor steht)?
		brne	kein_kick			; wenn nicht  x springen
		push	w					; w sichern 
		push	w2					; w2 sichern
		lds		FSzaehler,KickZeit	; Kickstartzeit setzen
		lds		w,KickFS			; w mit Kickstart FS laden
		sbrc	DCCReg,nutzeKennl	; wenn nicht nach Kennline in CV67-94 1xspringen
		rjmp	PC+2				; sonst mit V_start vorbelegen berspringen
		lds		V_ist,V_start		; V_start holen und zur sicherheit auch noch			
		inc		V_ist				; V_ist 1x inc
		rjmp	FS_kick_setzen		; und gleich ausgeben
kein_kick:
		lds		FSzaehler,IncWert	; bescheunigungs wert laden
		inc		V_ist				; V_ist erhhen
		rjmp	FS_setzen			; und Fahrstufe setzen
V_bremsen:							; hier abbremsen
		lds		FSzaehler,DecWert	; verzgerungs wert laden
		dec		V_ist				; V_ist verringern
		rjmp	FS_setzen			; und Fahrstufe setzen
V_ist_konst:						; Gleichbleibende Geschwindigkeit
		inc		FSzaehler			; wieder auf1 -> in 2,56ms erneut hier um auf nderungen zu testen
		rjmp	TIM2_ENDE			; und fertig
FS_setzen:							; Timerregister mit neuer FS setzen	
		push	w					; w sichern 
		push	w2					; w2 sichern
		mov		w,V_ist				; V_ist nach w kopieren	
FS_kick_setzen:						; hier landet Kickstarteinsprung
		sbrs	Flag,PWM9Bit		; wenn 9 Bit PWM 1x springen
		rjmp	PC+5				; wenn nicht (8-bit PWM) 5x springen
		clr		w2					; w2 lschen
		lsl		w					; V_ist verdoppeln dazu *2 (aus 8 9 bit machen fr 16kHz )
		rol		w2					; bertrag in w2 einschieben	
		out		OCR1AH,w2			; high Byte (bit9) der PWM Impulsbreite setzen

		out		OCR1AL,w			; PWM Impulsbreite setzen (bzw low Byte bei 9bit)
		in		w,TCCR1A
    	sbr		w,(1<<COM1A1)		; PWM Ausgang verbinden
		out		TCCR1A,w
		pop		w2					; w2 wiederherstellen
		pop		w					; w wiederherstellen

TIM2_ENDE:
		out		sreg,intsreg		; sreg wiederherstellen
		reti						; und fertig

;************************************************************************************************
;*						Unterprogramme															*
;************************************************************************************************
.include "init_m8lokdec.inc"		; init des mega8
.include "DCC.inc"					; DCC Informationen einlesen
.include "Eprom.inc"				; liest oder schreibt ein Byte aus eead in w
.include "Funktionsbefehle.inc"		; setzt die Funktionsausgnge
.include "Fahrbefehle.inc"			; setzt die Fahrbefehle (Geschwindigkeit + Richtung)
.include "Programmierung.inc"		; Programmiert den Dekoder
;.include "RS232_m8.inc"				; UART fr m8
;.include "ADC_m8.inc"				; ADC fr m8


;************************************************************************************************
;*						Hauptprogramm															*
;************************************************************************************************

Reset:	ldi		w,high(RAMEND)		; StackPointer setzen
        out 	SPH,w
        ldi 	w,low(RAMEND)
        out 	SPL,w
		
		cli							; interrups ausschalten
; Ports setzen
		ldi  	w,0b00000111		; PB0-2 als Ausgang
	  	out  	DDRB,w
		ldi		w,0b11111000		; alle Portpins auf low
		out		PORTB,w
		ldi		w,0b00000000		; PC als Eingang
		out		DDRC,w
		ldi		w,0b11111110		; auer PC0 alle Pullups ein
		out		PORTC,w
		ldi  	w,0b11111010		; PD1 (TXD),d3-7 als Ausgang Rest als Eingang
	  	out  	DDRD,w
		ldi		w,0b00000011		; alle Pullup ein, TXD auf high
		out		PORTD,w

; register laden

		clr		w					; EEProm high Addresse lschen
		out		EEARH,w
		clr		V_ist
		clr		V_soll
		clr		lowpegel			
        clr		highpegel
		clr		Flag
		clr		Zaehler
		CLR		FSzaehler
        ldi     zl,LOW(EMK)  		; den Z-Pointer mit dem Start der Generator Bytes laden
        ldi     zh,HIGH(EMK)

.include "LoadEEdaten.inc"			; Daten aus EEProm einlesen

; Hardware init

;		rcall	init_uart_m8		; UART init
;		rcall	init_adc_m8			; ADC init
		rcall	init_timer1_m8		; Timer1 init 
		rcall	init_timer2_m8		; Timer2 init -> 10us CTC interrupt zum DCC Signal einlesen

		sei							; interrups einschalten

Hauptschleife:
		rcall	DCC_schleife		; auf neue DCC Daten warten
									; neue Daten prfen und bergabe


;		sbrc 	Flag,resetbit		; wenn 1 war reset da gehe zu
;		rjmp	progmode			; Programmiermodus CV-Lesen, Schreiben, Bitmanipulation

					 				; prfe Adresse auf 0
  		tst 	byteA				; wenn 0 dann 
 		breq 	an_alle				; Broadcast an alle
	  
  		ldi 	w,$FF				; prfe Adresse auf $FF
  		cp 		byteA,w				; 
  		brne	PC+2				; wenn ungleich 1x springen
Leerlauf:							; wenn gleich Leerlaufpaket -> nichts machen bei diesem befehl)
  	  	rjmp	Hauptschleife
			  
  		cp  	byteA,Adresse		; wenn eigene Adresse
  		breq 	an_mich				; dann test OK
	  	  
 	  	rjmp	Hauptschleife		; sonst Hauptschleife

an_alle: 							; alle Decoder sind angesprochen
an_mich:							; eigene Adresse ist angesprochen
;hier fehlt noch auf lange Adresse testen und verzweigen
;hier je nach Datenformat verzweigen
  		mov 	w,byteB				; Daten sichern
		cpi		w,0b00000000		; Vergleich auf dcc reset
		brne	PC+2				; wenn ungleich 1x springen
		rjmp	Reset				; sonst Reset machen
		andi 	w,0b11100000	 	; Datenbits ausblenden
	  	cpi 	w,0b00100000		; Vergleiche auf erweiterte Operation (enthlt befehl fr 128 Fahrstufen)
	  	brne 	PC+2				; wenn ungleich 1x springen
		rjmp	erw_Operation		; wenn gleich gehe zu..
  		cpi 	w,0b01000000		; Vergleiche mit V Rckwrts einfaches Format
		brne	PC+3				; wenn ungleich 2x springen
  		mov 	w,byteB				; w mit Daten laden
  		rjmp 	V_Basis  			; wenn ja gehe zu
  		cpi 	w,0b01100000		; Vergleiche mit V Vorwrts einfaches Format
		brne	PC+3				; wenn ungleich 2x springen
  		mov 	w,byteB				; w mit Daten laden
  		rjmp 	V_Basis				; wenn ja gehe zu..
  		cpi 	w,0b10000000		; Vergleiche mit Funktionsgruppe F1-F4
		brne	PC+3				; wenn ungleich 2x springen
  		mov 	w,byteB				; w mit Funktionsdaten laden
  		rjmp 	Fgruppe1			; und dann gehe zu..
		cpi		w,0b11100000		; Vergleich auf CV reg Manipulation
		brne	PC+2				; wenn ungleich 1x springen
		rjmp	CV_edit				; wenn gleich gehe zu..
										; alles andere wird nicht unterstzt -> verwerfen und zurck
  		rjmp 	Hauptschleife		; Endlosschleife

.ESEG

;CV Register im eeprom vom Lokdec

.db 1			;CV1 Dekoderaddresse 1-127
.db 5			;CV2 Minimale Geschwindigkeit 0-64
.db 20			;CV3 Anfahrverzgerung 1-256 1=2,56ms je Schritt 0(256)=655,35ms je Schritt
.db 5			;CV4 Bremsverzgerung 1-256 w.o.
.db 150			;CV5 Maximal-Geschwindigkeit in 6 Stufen 25% 50% 75% 100% 125% 150%
.db $FF			;CV6 	
.db 1			;CV7 Software-Version
.db $0D			;CV8 Herstellerkennung
.db	3			;CV9 Motor PWM-Periode 0-4 0=30Hz 1=120Hz 2=500Hz 3=16KHz 4=32KHz
.db $FF			;CV10
.db $FF			;CV11
.db $FF			;CV12
.db $FF			;CV13
.db $FF			;CV14
.db $FF			;CV15
.db $FF			;CV16
.db 192			;CV17 Erweiterte Adresse Hherwertiges Byte der langen Adresse plus 192
.db 0			;CV18 Erweiterte Adresse Niederwertiges Byte der langen Adresse
.db $FF			;CV19 Consist-Adresse wird nicht untersttzt!
.db $FF			;CV20
.db $FF			;CV21
.db $FF			;CV22
.db $FF			;CV23
.db $FF			;CV24
.db $FF			;CV25
.db $FF			;CV26
.db $FF			;CV27
.db $FF			;CV28
.db 2			;CV29 Konfiguration nach DCC-Norm
				;		+1:Fahr-Richtung umgekehrt
				;		+2:28/128 Fahrstufen, sonst 14/27
				;		+4:automatisch Analog/Digital-Erkennung, sonst nur Digitalbetrieb
				;		+8:letzte Geschwindigkeit+Richtung im EEprom merken
                ;		+16:Kennlinie auf CV66-95, sonst Fahrstufen ber CV2, CV5 und CV6
                ;		+32:Erweiterte (lange) Adresse (CV17 und CV18), sonst CV1
				;		+128:Zubehrdekoder, sonst Lokdekoder
;CV30-48
.db $FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF
.db $FF			;CV49 - CV64 Herstellerparameter
.db	5			;CV50 Dimmen der Lichtausgnge 0-10
.db	0			;CV51 Vmerken merke letzte Geschwindigkeit+Richtung
;CV52-64
.db $FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF
.db	20			;CV65 Kickstartzeit 1-256 1=2,56ms 0(256)=655,35ms lang Vmax beim anfahren
.db $7F			;CV66 KickstartSpannung 0=aus 255=Vmax
.db 10			;CV67 - 94
.db	20			; alternative Geschwindigkeitkennlinie. (nur bei Modus 28 Fahrstufen)
.db 28			; Jeder der 28 Fahrstufen wird eine spezifische Motorspannung zugeordnet
.db 36			; mindestwert ist 0 (Motor aus) und 255 maximale Motorspannung 
.db 44			;CV74 Fahrstufe 05 
.db 48			;CV74 Fahrstufe 06  
.db 56			;CV74 Fahrstufe 07 
.db 64			;CV74 Fahrstufe 08
.db 72			;CV75 Fahrstufe 09
.db 80			;CV76 Fahrstufe 10
.db 88			;CV77 Fahrstufe 11
.db 96			;CV78 Fahrstufe 12
.db 104			;CV79 Fahrstufe 13
.db 112			;CV80 Fahrstufe 14
.db 120			;CV81 Fahrstufe 15
.db 128			;CV82 Fahrstufe 16
.db 136			;CV83 Fahrstufe 17
.db 144			;CV84 Fahrstufe 18
.db 152			;CV85 Fahrstufe 19
.db 160			;CV86 Fahrstufe 20
.db 168			;CV87 Fahrstufe 21
.db 176			;CV88 Fahrstufe 22
.db 184			;CV89 Fahrstufe 23
.db 192			;CV90 Fahrstufe 24
.db 200			;CV91 Fahrstufe 25
.db 208			;CV92 Fahrstufe 26
.db 216			;CV93 Fahrstufe 27
.db 224			;CV94 Fahrstufe 28
